# sleepyHead

A wicked smart jQuery image/video lazy-load plugin.

## Usage
Implementing sleepyHead requires a tiny bit of HTML and JS.

#### HTML
Add a `data-src` attribute to all img tags you wish to lazy load. When appropriate, the image path in the data-src attribute will replace the value in the src attribute.

	<img src="/image.jpg" data-src="/image.png" height="500" width="500" />

To load retina specific images use the `data-src-retina` attribute.

#### JS
On document ready, select your images, and call the sleepyHead( ) method.

	$(document).ready(function() {

		$('img').sleepyHead(yOffset, xOffset);

	});



### yOffset & xOffset params (optional)
Images do not load instantaneously, and without specifying these params, the user might see a empty image for a split second while scrolling the page.

To remedy this, I like to call sleepyHead as such:

	$('img').sleepyHead(200, 400);

The yOffset param will load images 200px before they enter the users viewport.

For sites that use image carousels/sliders, specifying an xOffset param can be helpful to preload the next couple of images, before they come into view.

### Carousel/slider Image Loading

Independent of the framework, there are many ways to change slides when using a carousel:

* Click events
* Touch events
* Timers

To lazy load carousel images, simply trigger a `sleepyHead` event on the `document` after every slide change, and things should work marvelously.

Using [FlexSlider](http://github.com/woothemes/flexslider), I trigger the event using the *after* method:

	felxSlider: function() {
		var self = this;
		$('selector').flexSlider({
		  animation: 'slide',
		  slideshow: true,
		  after: function() {
        	$(document).trigger('sleepyHead');
		  }
		});
	},

### Loading Video
sleepyHead will lazy load embedded media in addition to images.

#### HTML
	<iframe
		data-src="http://player.vimeo.com/video/43311073"
		width="500"
		height="281"
		frameborder="0">
	</iframe>

#### JS
	$('iframe').sleepyHead();


## Appendix

#### Maintenance
If you encounter any bugs, or have ways to improve sleepyHead, fork and pull, or just shoot me an email at jay@moonfuse.co.